/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.elementLibrary.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.dto.elementLibrary.*;

import java.util.List;

/**
 * 元素目录管理Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuElementGroupApiService extends ExcelService
{
  /**
   * 查询元素目录管理列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AtuElementGroupDetailDto> pagingQuery(QueryByPage<AtuElementGroupQueryDto> queryByPage);

  /**
   * 新增元素目录管理
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(AtuElementGroupAddDto dto);

  /**
   * 更新元素目录管理
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuElementGroupUpdateDto dto) throws IllegalAccessException;

  /**
   * 删除元素目录管理
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

  /**
   * 根据层级查询元素库目录节点
   * @param dto
   * @return
   */
  List<AtuElementGroupDetailDto> queryElementTreeByParentId(AtuElementGroupQueryDto dto);

  /**
   * 根据名字模糊查询元素库目录节点
   * @param dto
   * @return
   */
  List<AtuElementGroupDetailDto> queryElementTreeByName(AtuElementGroupQueryDto dto);

  /**
   * 拖拽元素目录树
   * @param dto
   * @return
   */
    Integer moveElementTree(AtuElementGroupMoveDto dto);
}
